function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/** Virtual list, only renders visible items.
 *  @param {Array<*>} data         List of data items
 *  @param {Function} renderRow    Renders a single row
 *  @param {Number} rowHeight      Static height of a row
 *  @param {Number} overscanCount  Amount of rows to render above and below visible area of the list
 *  @example
 *  new VirtualList(element, {
 *    data: ['a', 'b', 'c'],
 *    renderRow: (row) => { <div>{row}</div> },
 *    rowHeight: 150
 *  )}
 *
 *  TODO:
 *  – Add support for variable heights
 *  – Add a scroll to index method
 *  – Add an initial scroll offset or index option
 */

var STYLE_INNER = 'position:relative; overflow:hidden; width:100%; min-height:100%;';
var STYLE_CONTENT = 'position:absolute; top:0; left:0; height:100%; width:100%; overflow:visible;';

var VirtualizedList = function () {
  function VirtualizedList(container, options) {
    _classCallCheck(this, VirtualizedList);

    this.container = container;
    this.options = options;

    // Initialization
    this.state = {};

    var inner = this.inner = document.createElement('div');
    var content = this.content = document.createElement('div');

    inner.appendChild(content);
    container.appendChild(inner);

    // Binding
    this.render = this.render.bind(this);
    this.handleScroll = this.handleScroll.bind(this);
    this.resize = this.resize.bind(this);

    // Resize
    this.resize();

    // Add event listeners
    this.container.addEventListener('scroll', this.handleScroll);
    window.addEventListener('resize', this.resize);
  }

  VirtualizedList.prototype.setState = function setState() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this.state = Object.assign(this.state, state);

    window.requestAnimationFrame(this.render);
  };

  VirtualizedList.prototype.resize = function resize() {
    if (this.state.height !== this.container.offsetHeight) {
      this.setState({
        height: this.container.offsetHeight
      });
    }
  };

  VirtualizedList.prototype.handleScroll = function handleScroll() {
    this.setState({
      offset: this.container.scrollTop
    });
  };

  VirtualizedList.prototype.destroy = function destroy() {
    window.removeEventListener('resize', this.resize);

    // TODO: Destroy everything properly, silly
  };

  VirtualizedList.prototype.render = function render() {
    var _options = this.options,
        data = _options.data,
        rowHeight = _options.rowHeight,
        renderRow = _options.renderRow,
        _options$overscanCoun = _options.overscanCount,
        overscanCount = _options$overscanCoun === undefined ? 3 : _options$overscanCoun;
    var _state = this.state,
        _state$offset = _state.offset,
        offset = _state$offset === undefined ? 0 : _state$offset,
        _state$height = _state.height,
        height = _state$height === undefined ? 0 : _state$height;

    // first visible row index

    var start = offset / rowHeight | 0;

    // actual number of visible rows (without overscan)
    var visibleRowCount = height / rowHeight | 0;

    // Overscan: render blocks of rows modulo an overscan row count
    // This dramatically reduces DOM writes during scrolling
    if (overscanCount) {
      start = Math.max(0, start - start % overscanCount);
      visibleRowCount += overscanCount;
    }

    // last visible + overscan row index
    var end = start + 1 + visibleRowCount;

    // data slice currently in viewport plus overscan items
    var selection = data.slice(start, end);

    var fragment = document.createDocumentFragment();
    selection.forEach(function (row) {
      return fragment.append(renderRow(row));
    });

    this.inner.setAttribute('style', STYLE_INNER + ' height:' + data.length * rowHeight + 'px;');
    this.content.setAttribute('style', STYLE_CONTENT + ' top:' + start * rowHeight + 'px;');

    this.content.innerHTML = '';
    this.content.appendChild(fragment);
  };

  return VirtualizedList;
}();

export { VirtualizedList as default };